"""
Copyright 2023 Fausto Oliveira
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import logging
from typing import Dict, List

from mkdocs.config import base
from mkdocs.config import config_options as c
from mkdocs.plugins import BasePlugin, Navigation, Page

from .content_manager import ContentManager
from .page_parser import PageParser
from .section_reader import SectionReader

LOG = logging.getLogger("mkdocs.plugins." + __name__)

DEFAULT_METADATA_NAME = 'childpages_card_grid'
TITLE_METADATA_NAME = DEFAULT_METADATA_NAME + '_title'


class ChildPagesCardGridPluginConfig(base.Config):
    """
    Plugin configuration class
    """
    include_all = c.Type(bool, default=True)
    cards_title = c.Type(str, default='Child pages')


class ChildPagesCardGridPlugin(BasePlugin[ChildPagesCardGridPluginConfig]):
    """
    The actual plugin class - see README.md in the root directory for what the plugin does
    """

    # prevent reading the navigation map over and over again
    read = False

    # contains a map of child pages
    nav_map: Dict[str, List[Page]] = dict()

    # manages the html manipulation
    content_manager = ContentManager()

    # read the navigation map
    section_reader = SectionReader()

    def __init__(self):
        self.enabled = True

    def on_pre_build(self, *, config) -> None:
        """
        Clean the navigation map to read it again before every build
        """
        del config

        self.nav_map.clear()
        self.read = False

    def on_page_context(self, context, *, page, config, nav: Navigation):
        """
        Populate the navigation map
        """
        del page, config

        if not self.read:
            for item in nav:
                self.read_item(item)

        self.read = True

        return context

    def read_item(self, item):
        """
        Recursively read items from the navigation object
        """
        if item:
            if type(item).__name__ == "Section":
                self.section_reader.read_section(item, self.nav_map)

            if item.children:
                for child in item.children:
                    self.read_item(child)

    def on_post_page(self, output: str, *, page: Page, config):
        """
        Insert the children grid card into the page after the main tag
        """
        del config

        # add all children by default, or not, based on plugin config
        add_card_grid = self.config.include_all

        if DEFAULT_METADATA_NAME in page.meta:
            metadata_value = page.meta[DEFAULT_METADATA_NAME].lower()
            # set to false if metadata says to exclude
            if metadata_value == 'exclude':
                add_card_grid = False
            # set to True if metadata says to include
            if metadata_value == 'include':
                add_card_grid = True

        cards_title = self.config.cards_title
        LOG.info('page.title=%s', page.title)
        LOG.info('TITLE_METADATA_NAME=%s', TITLE_METADATA_NAME)
        if TITLE_METADATA_NAME in page.meta:
            cards_title = page.meta[TITLE_METADATA_NAME]
        LOG.info('cards_title=%s', cards_title)

        if add_card_grid and page.file.dest_uri in self.nav_map:
            child_list = self.nav_map[page.file.dest_uri]
            if len(child_list) > 0:
                # find the main tag in the output html
                page_parser = PageParser()
                page_parser.feed(output)

                if page_parser.article_closing_tag_location:
                    new_content = self.content_manager.generate_cards_div(
                        page, cards_title, child_list)
                    new_output = self.content_manager.insert_new_content(
                        output, new_content, page_parser.article_closing_tag_location)
                    return new_output

        return output



"""
Copyright 2023 Fausto Oliveira
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from html.parser import HTMLParser
import logging
import dataclasses

LOG = logging.getLogger("mkdocs.plugins." + __name__)


@dataclasses.dataclass
class ArticleClosingTagLocation:
    """
    Represents the location of the closing </article> tag in an HTML document.
    """

    line: -1
    offset: -1

    def __init__(self, line: int, offset: int):
        self.line = line
        self.offset = offset


class PageParser(HTMLParser):
    """
    HTML Parser to read an HTML document and locate the closing </article> tag.
    """

    article_closing_tag_location: ArticleClosingTagLocation

    def handle_endtag(self, tag):
        if tag == "article":
            line, offset = self.getpos()
            self.article_closing_tag_location = ArticleClosingTagLocation(
                line, offset)

    def error(self, message: str):
        return LOG.error(message)



"""
Copyright 2023 Fausto Oliveira
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
from typing import List
from mkdocs.plugins import Page

from .page_parser import ArticleClosingTagLocation


class ContentManager:
    """
    Manage the HTML contents of the pages that are being modified by this plugin
    """

    def generate_cards_div(self, page: Page, title: str, child_list: List[Page]) -> str:
        """
        Generate the div containing all cards
        """
        new_content = f'<div class="card_title">{title}</div>'
        new_content += '<div class="cards">'
        for child in child_list:
            new_content += f'<a href="{child.file.url_relative_to(page.file)}">'
            new_content += '<div class="card"><div class="card_container">'
            new_content += f'<p>{child.title}</p>'
            new_content += '</div></div></a>'
            new_content += os.linesep
        new_content += '</div>'
        return new_content

    def insert_new_content(self, content: str, new_content: str,
                           article_closing_tag_location: ArticleClosingTagLocation) -> str:
        """
        Takes a content string and adds the new_content to it in the correct position,
        given the article_closing_tag_location
        """
        content_lines = content.splitlines()

        line_str = content_lines[article_closing_tag_location.line - 1]
        being_str = line_str[:article_closing_tag_location.offset - 1]
        end_str = line_str[article_closing_tag_location.offset:]
        content_lines[article_closing_tag_location.line -
                      1] = being_str + new_content + end_str

        new_output = os.linesep.join(content_lines)

        return new_output



import os
from setuptools import find_packages, setup

# --------------------
# Setup
# --------------------

def read_file(fname):
    "Read a local file"
    return open(file=os.path.join(os.path.dirname(__file__), fname), encoding="utf-8").read()

setup(
    name='mkdocs-childpages-card-grid-plugin',
    version=read_file('VERSION').strip(),
    description='Create card grids with links to child pages on your MkDocs site',
    long_description=read_file('README.md'),
    long_description_content_type='text/markdown',
    keywords=["mkdocs", "plugin", "python", "child-pages", "card-grid"],
    url='https://github.com/faustool/mkdocs-childpages-card-grid-plugin',
    author='Fausto Oliveira',
    license='Apache License, Version 2.0',
    python_requires='>=3.7',
    install_requires=[
        'mkdocs>=1.4.0',
        "mkdocs-material>=8.5.0"
    ],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3'
    ],
    include_package_data=True,
    packages=find_packages(exclude=['test']),
    entry_points={
        'mkdocs.plugins': [
            'childpages-card-grid = childpages_card_grid.plugin:ChildPagesCardGridPlugin'
        ]
    }
)



"""
Copyright 2023 Fausto Oliveira
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import logging

from typing import Dict, List
from mkdocs.plugins import Page

LOG = logging.getLogger("mkdocs.plugins." + __name__)


class SectionReader:
    """
    Read child sections from a navigation item and populate the nav_map dictionary for use
    when rendering pages.
    This function expectes that the first child of every section represents the page itself and
    the subsequent children represent the actual page children.
    The scenario below illustrates what this means:
    Section A - this section
        Page - the page representing Section A
        Page - a child page of Section A
        Section B - a child section of Section A
            Page - the page representing Section B, therefore a child page of Section *A*
            Page - a child page of Section B
        Page - another child page of Section A
        etc.
    """

    def read_section(self, section, nav_map: Dict[str, List[Page]]):
        """
        Read the section
        """
        if isinstance(section.children[0], Page):
            parent_page: Page = section.children[0]  # first child is a page
            LOG.debug("Parent: %s: %s", parent_page.title,
                      parent_page.file.dest_uri)
            # init the child list for this page
            nav_map[parent_page.file.dest_uri] = []
            # iterate over the subsequent children
            for i in range(1, len(section.children)):
                # whe don't know waht this child is
                child_candidate = section.children[i]
                child_page = None
                if isinstance(child_candidate, Page):  # is it a page?
                    child_page = child_candidate  # the child page is the child itself
                # is the candidate a section? does it have a child and is the child a page?
                elif type(child_candidate).__name__ == "Section" \
                        and child_candidate.children \
                        and isinstance(child_candidate.children[0], Page):
                    # the page is the first child of the section
                    child_page = child_candidate.children[0]
                LOG.debug("  Child: %s: %s", child_page.title,
                          child_page.file.dest_uri)
                # insert the child page to the navigation map
                nav_map[parent_page.file.dest_uri].append(child_page)
